#!/bin/bash

#     Copyright (C) 2009 Michael Verret <michael.verret@gmail.com>
#
#            This script is released under the terms of the:
#                      GNU GENERAL PUBLIC LICENSE
#                        Version 3, 29 June 2007
#                http://www.gnu.org/licenses/gpl-3.0.txt
#
# This is a Bash shell script written to automate the process of
# generating software packages suitable for use with Zenwalk Linux.
#
# Zenwalk Linux is a GNU/Linux operating system.
#
#              For more information on Zenwalk Linux visit:
#                        http://www.zenwalk.org/
#
# Bash is a free software Unix shell written for the GNU Project.
#
#                   For more information on Bash visit:
#           http://tiswww.case.edu/php/chet/bash/bashtop.html


# Package variables:
#_______________________________________________________________________
Summary="NV display driver"
Driver_Type="video"
Source_Module="nv"
Source_Category="driver"
Source_Version="2.1.13"
Source_Format="tar.bz2"
Flag_Arch="i686"
Package_Arch="${Flag_Arch}"
Package_Release="62"
Package_Build="1"
Source_Name="xf86-${Driver_Type}-${Source_Module}"
Source_URL="http://xorg.freedesktop.org/releases/individual/${Source_Category}/"
Package_Name="xorg-${Source_Category}-${Source_Module}"
Package_Version="${Source_Version}"
File_Image=" "
File_Misc=" \
	${Source_Module}.xinf"
File_Patch=" \
	nv-2.1.12-gf7025-gf7050.patch.gz \
	nv-2.1.6-panel-fix.patch.gz \
	nv-2.1.6-starvation.patch.gz \
	nv-2.1.8-g80-no-doublescan.patch.gz \
	nv-reserve-fbarea.patch.gz \
	nv-save-rom.patch.gz"
File_Script=" \
	build-${Package_Name}.sh"
Special_Docs=" "
Desc_Line_00=": (${Summary})"
Desc_Line_01=":"
Desc_Line_02=": This driver for the X.Org X server provides support for."
Desc_Line_03=": NVIDIA Riva, TNT, GeForce, and Quadro cards. Note that"
Desc_Line_04=": this is not the same as the binary-only 'nvidia' driver,"
Desc_Line_05=": which adds 3D support, but is binary-only and is not"
Desc_Line_06=": opensource software"
Desc_Line_07=":"
Desc_Line_08=":"
Desc_Line_09=":"
Desc_Line_10=":"
# BuildRequires: 
# Requires:


# Constant variables:
#_______________________________________________________________________
Build_Root=$(pwd)
Source_Duo="${Source_Name}-${Source_Version}"
Source_Title="${Source_Duo}.${Source_Format}"
Package_Duo="${Package_Name}-${Package_Version}"
Package_Title="${Package_Duo}-${Package_Arch}-${Package_Release}.${Package_Build}"
Package_Root="${Build_Root}/${Package_Title}"
Standard_Docs="AUTHORS BUGS COPYING ChangeLog INSTALL LICENSE NEWS README"
Custom_Files="${File_Image} ${File_Misc} ${File_Patch} ${File_Script}"


# Build & install:
#_______________________________________________________________________
for Each in ${Source_Duo} ${Package_Name}-${Package_Version}-${Package_Arch} ; do
	if [ -d ${Each} ] ; then
		rm -rf ${Each}
	fi
done
for Each in ${Package_Title}.dep ${Package_Title}.md5 ${Package_Title}.src.md5 \
	${Package_Title}.src.tbz ${Package_Title}.tgz ; do
	if [ -e ${Each} ] ; then
		rm -rf ${Each}
	fi
done
if [ -e ${Source_Title} ] ; then
	echo "${Source_Title} already downloaded"
else
	wget -c -t 0 -T 50 -w 0 ${Source_URL}${Source_Title}
fi

for Archive in ${Source_Title} ; do
	File_Type=$(file -biz "${Archive}")
	unset Command
	case "${File_Type}" in
		*application/x-tar*)
			Command="tar -xf" ;;
		*application/x-zip*)
			Command="unzip" ;;
		*application/x-gzip*)
			Command="gunzip -d -f" ;;
		*application/x-bzip*)
			Command="bunzip2 -f" ;;
	esac
	if [ "$Command" != "" ]; then
		echo "${Command} ${Archive}"
		${Command} ${Archive} || exit 1
	fi
done

cd ${Source_Duo}

chown -R root:root .
find . -perm 666 -exec chmod 644 {} \;
find . -perm 664 -exec chmod 644 {} \;
find . -perm 600 -exec chmod 644 {} \;
find . -perm 444 -exec chmod 644 {} \;
find . -perm 400 -exec chmod 644 {} \;
find . -perm 440 -exec chmod 644 {} \;
find . -perm 777 -exec chmod 755 {} \;
find . -perm 775 -exec chmod 755 {} \;
find . -perm 511 -exec chmod 755 {} \;
find . -perm 711 -exec chmod 755 {} \;
find . -perm 555 -exec chmod 755 {} \;

zcat ${Build_Root}/nv-reserve-fbarea.patch.gz | patch -p1 -b --suffix=.reserve-fbarea || exit 1
zcat ${Build_Root}/nv-2.1.6-starvation.patch.gz | patch -p1 -b --suffix=.starve || exit 1
zcat ${Build_Root}/nv-2.1.6-panel-fix.patch.gz | patch -p1 -b --suffix=.panel || exit 1
zcat ${Build_Root}/nv-save-rom.patch.gz | patch -p1 -b --suffix=.save-rom || exit 1
zcat ${Build_Root}/nv-2.1.8-g80-no-doublescan.patch.gz | patch -p1 -b --suffix=.doublescan || exit 1
zcat ${Build_Root}/nv-2.1.12-gf7025-gf7050.patch.gz | patch -p1 -b --suffix=.nv6x || exit 1

autoreconf -iv

export CFLAGS="-O2 -march=${Flag_Arch} -pipe"
export CXXFLAGS="${CFLAGS}" 
export PKG_CONFIG_PATH=/usr/lib/pkgconfig
./configure \
--prefix=/usr \
--libdir=/usr/lib \
--mandir=/usr/man \
--disable-static \
--silent

make --silent || exit 1
make install --silent  DESTDIR=${Package_Root} || exit 1


# Special Routines:
#_______________________________________________________________________
# Install hardware identification / configuration data:
mkdir -p ${Package_Root}/usr/share/hwdata/videoaliases || exit 1
install -m 0644 ${Build_Root}/${Source_Module}.xinf ${Package_Root}/usr/share/hwdata/videoaliases/ || exit 1


# Complete the final packaging routines:
#_______________________________________________________________________
mkdir -p ${Package_Root}/usr/doc/${Package_Duo}/
for Each_Doc in {${Special_Docs},${Standard_Docs}} ; do
	if [ -e ${Each_Doc} ] ; then
		cp ${Each_Doc} ${Package_Root}/usr/doc/${Package_Duo}/
	fi
done

( cd ${Package_Root}/usr/doc/${Package_Duo}/
	find . -name "*" -size 0c -exec rm {} \;
)

cd ${Build_Root}
rm -rf ${Source_Duo}

mkdir ${Package_Root}/install
cat <<EOF0 > ${Package_Root}/install/slack-desc
${Package_Name}${Desc_Line_00}
${Package_Name}${Desc_Line_01}
${Package_Name}${Desc_Line_02}
${Package_Name}${Desc_Line_03}
${Package_Name}${Desc_Line_04}
${Package_Name}${Desc_Line_05}
${Package_Name}${Desc_Line_06}
${Package_Name}${Desc_Line_07}
${Package_Name}${Desc_Line_08}
${Package_Name}${Desc_Line_09}
${Package_Name}${Desc_Line_10}
EOF0

mkdir -p ${Package_Root}/usr/src/${Package_Duo}
for Each_Custom in ${Custom_Files} ; do
	if [ -e ${Each_File} ]; then
		cp ${Each_Custom} \
			${Package_Root}/usr/src/${Package_Duo}/
	fi
done

( cd ${Package_Root}
	find . | xargs file | grep "executable" | grep ELF | cut -f 1 -d : | xargs strip --strip-unneeded 2> /dev/null
	find . | xargs file | grep "shared object" | grep ELF | cut -f 1 -d : | xargs strip --strip-unneeded 2> /dev/null
)

if [ -d ${Package_Root}/usr/man ]; then
	( cd ${Package_Root}/usr/man
		gzip -9 */*
	)
fi

( cd ${Package_Root}
	requiredbuilder -z ${Package_Root} > ${Build_Root}/${Package_Title}.dep
)

( cd ${Package_Root}
	makepkg -l y -c n ${Build_Root}/${Package_Title}.tgz
)
cd ${Build_Root}
md5sum ${Package_Title}.tgz > ${Package_Title}.md5
rm -rf ${Package_Root}

mkdir -p ${Package_Root}.src
cp ${Source_Title} ${Package_Root}.src
for Each_Set in ${Custom_Files} ; do
	if [ -e ${Each_Set} ] ; then
		cp ${Each_Set} ${Package_Root}.src
	fi
done
tar -jcf ${Package_Title}.src.tbz ${Package_Title}.src
rm -rf ${Package_Title}.src 
md5sum ${Package_Title}.src.tbz > ${Package_Title}.src.md5


# Changelog of script edits:
#_______________________________________________________________________


# ~ Thu Apr 16th 2009	Michael Verret "axxium" <michael.verret@gmail.com>
# ~ Software: xorg-driver-nv-2.1.13
# ~ Build 62.1
# ~ 	Removed the following patches:
# ~ 	nv-1.1.1-nv34-tweaks.patch
# ~ 	nv-2.1.12-eedid.patch

# ~ Fri Mar 13th 2009	Michael Verret "axxium" <michael.verret@gmail.com>
# ~ Software: xorg-driver-nv-2.1.12
# ~ Build 62.2

# ~ Wed Mar 4th 2009	Michael Verret "axxium" <michael.verret@gmail.com>
# ~ Software: xorg-driver-nv-2.1.12
# ~ Build 62.1

# ~ Thu Feb 12th 2009	Michael Verret "axxium" <michael.verret@gmail.com>
# ~ Software: xorg-driver-nv-2.1.12
# ~ Build 60.2

# ~ Wed Feb 4th 2009	Michael Verret "axxium" <michael.verret@gmail.com>
# ~ Software: xorg-driver-nv-2.1.12
# ~ Build 60.1
